function [D] = DL2_kNN_k_estimation(X,Y,co)
%Estimates the L2 divergence (D) of X and Y (X(:,t), Y(:,t) is the t^th sample)
%using the kNN method (S={k}). The number of samples in X [=size(X,2)] and Y [=size(Y,2)] can be different. Cost parameters are provided in the cost object co.
%
%We make use of the naming convention 'D<name>_estimation', to ease embedding new divergence estimation methods.
%
%REFERENCE: 
%   Barnabs Pczos, Zoltn Szab, Jeff Schneider: Nonparametric divergence estimators for Independent Subspace Analysis. EUSIPCO-2011, pages 1849-1853.
%   Barnabs Pczos, Liang Xiong, Jeff Schneider. Nonparametric Divergence: Estimation with Applications to Machine Learning on Distributions. UAI-2011.
%
%Copyright (C) 2012 Zoltan Szabo ("http://nipg.inf.elte.hu/szzoli", "szzoli (at) cs (dot) elte (dot) hu")
%
%This file is part of the ITE (Information Theoretical Estimators) Matlab/Octave toolbox.
%
%ITE is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with ITE. If not, see <http://www.gnu.org/licenses/>.

%co.mult:OK.

[dY,num_of_samplesY] = size(Y);
[dX,num_of_samplesX] = size(X);

if dX~=dY
    disp('Error: the dimension of X and Y must be equal.');
else
    d = dX;
end

squared_distancesXX = kNN_squared_distances(X,X,co,1);
squared_distancesYX = kNN_squared_distances(Y,X,co,0);
dist_k_XX = sqrt(squared_distancesXX(end,:));
dist_k_YX = sqrt(squared_distancesYX(end,:));

c = pi^(d/2) * gamma(d/2+1);

term1 = (co.k-1) / ((num_of_samplesX-1)*c) ./ dist_k_XX.^d;
term2 = 2 * (co.k-1) / (num_of_samplesY*c) ./ dist_k_YX.^d;
term3 = (num_of_samplesX - 1) * c  * (co.k-2) * (co.k-1) / (co.k * (num_of_samplesY*c)^2) * (dist_k_XX.^d ./ dist_k_YX.^(2*d));
L2 = mean(term1-term2+term3);
%D = sqrt(L2);%theoretically
D = sqrt(abs(L2));%due to the finite number of samples L2 can be negative. In this case: sqrt(L2) is complex; to avoid such values we take abs().
